﻿using System;
using System.Collections.Generic;
using gudusoft.gsqlparser;
using gudusoft.gsqlparser.Units;

namespace AnalyzePLSQLProcedure
{
    class analyzePLSQLProcedure
    {
        static void Main(string[] args)
        {
            TGSqlParser sqlparser = new TGSqlParser(TDbVendor.DbVOracle);
            sqlparser.SqlText.Text = "CREATE OR replace PROCEDURE Db_mdw_trt_elt_prel_cond ( p_param_call IN pkg_mdw_def_general.param_call, p_trt IN VARCHAR2 ) IS\n" +
                    "\n" +
                    "w_nom_ekapi VARCHAR2(255);\n" +
                    "w_msg pkg_mdw_def_general.param_msg;\n" +
                    "trt_error EXCEPTION;\n" +
                    "\n" +
                    "BEGIN\n" +
                    "\n" +
                    "w_nom_ekapi := 'PKG_EKI_ELT_PREL_COND';\n" +
                    "w_code_retour := p_code_retour;\n" +
                    "w_nom_proc := 'PKG_MDW_ELT_PREL_COND_ALIM.INIT_TCONDPREL';\n" +
                    "\n" +
                    "IF w_existe = 'N' THEN\n" +
                    "w_nom_proc := 'INSERT:POOLPREL';\n" +
                    "END IF;\n" +
                    "END db_mdw_trt_elt_prel_cond;";

            int ret = sqlparser.Parse();
            if (ret != 0)
            {
                Console.WriteLine(sqlparser.ErrorMessages);
                return;
            }

            TCustomSqlStatement sql = sqlparser.SqlStatements[0];
            Console.WriteLine("SQL Statement: " + sql.SqlStatementType);

            TPlsqlStatement stmt = (TPlsqlStatement)sql;
            if (stmt.SqlStatementType == TSqlStatementType.sstplsql_createprocedure)
            {
                TLzPlsql_SubProgram procedure = (TLzPlsql_SubProgram)stmt.Root;
                Console.WriteLine("Procedure name:  " + procedure._procedure_name.AsText);

                Console.WriteLine("Parameters:");

                TLzPlsql_ParameterDecl param = null;
                for (int i = 0; i < procedure._parameters.Count(); i++)
                {
                    param = (TLzPlsql_ParameterDecl)procedure._parameters[i];
                    Console.WriteLine("\tName:" + param._parameter_name.AsText);
                    Console.WriteLine("\tDatatype:" + param._plsql_datatype.AsText);
                    Console.WriteLine("\tIN/OUT:" + param._inouttype);
                }

                TLz_List declareStatements = procedure._decl_stmts;
                Console.WriteLine("declare statements: " + declareStatements.Count());
                TCustomSqlStatement declareStatement = null;
                TLzPlsql_VarDeclStmt variableDelcare = null;
                for (int i = 0; i < declareStatements.Count(); i++)
                {
                    declareStatement = (TCustomSqlStatement)declareStatements[i];
                    Console.WriteLine("SQL Statement: " + declareStatement.SqlStatementType);
                    variableDelcare = (TLzPlsql_VarDeclStmt)declareStatement;

                    switch (variableDelcare._declType)
                    {
                        case TDeclStmtType.dstVar:
                            Console.WriteLine("\tVariable Name:" + variableDelcare._varname.AsText);
                            Console.WriteLine("\tVariable Datatype:" + variableDelcare._typename.AsText);
                            break;
                        case TDeclStmtType.dstException:
                            Console.WriteLine("\tException:" + variableDelcare._varname.AsText);
                            break;

                    }
                }

                TLz_List bodyStatements = procedure._proc_sect;
                Console.WriteLine("body statements: " + bodyStatements.Count());
                TCustomSqlStatement bodyStatement = null;
                for (int i = 0; i < bodyStatements.Count(); i++)
                {
                    bodyStatement = (TCustomSqlStatement)bodyStatements[i];
                    Console.WriteLine("SQL Statement: " + bodyStatement.SqlStatementType);
                    switch (bodyStatement.SqlStatementType)
                    {
                        case TSqlStatementType.sstPlsql_AssignStmt:
                            TLzPlsql_AssignStmt assign = (TLzPlsql_AssignStmt)bodyStatement;
                            Console.WriteLine("left: " + assign._leftside.AsText);
                            Console.WriteLine("right: " + assign._rightside.AsText);
                            break;
                        case TSqlStatementType.sstPlsql_IfStmt:
                            TLzPlsql_IfStmt ifstmt = (TLzPlsql_IfStmt)bodyStatement;
                            Console.WriteLine("condition: " + ifstmt._condition.AsText);

                            if (ifstmt._then_stmt_list!=null && ifstmt._then_stmt_list.Count() > 0)
                            {
                                Console.WriteLine("then statement:");
                                for (int j = 0; j < ifstmt._then_stmt_list.Count(); j++)
                                {
                                    Console.WriteLine("\tStatement type: " + ((TCustomSqlStatement)ifstmt._then_stmt_list[j]).SqlStatementType);
                                    Console.WriteLine("\tStatement text: " + ((TCustomSqlStatement)ifstmt._then_stmt_list[j]).AsText);
                                }
                            }

                            if (ifstmt._elsif_list!=null && ifstmt._elsif_list.Count() > 0)
                            {
                                Console.WriteLine("else if statement:");
                                for (int j = 0; j < ifstmt._elsif_list.Count(); j++)
                                {
                                    Console.WriteLine("\tStatement type: " + ((TCustomSqlStatement)ifstmt._elsif_list[j]).SqlStatementType);
                                    Console.WriteLine("\tStatement text: " + ((TCustomSqlStatement)ifstmt._elsif_list[j]).AsText);
                                }
                            }

                            if (ifstmt._else_stmt_list!=null && ifstmt._else_stmt_list.Count() > 0)
                            {
                                Console.WriteLine("else statement:");
                                for (int j = 0; j < ifstmt._else_stmt_list.Count(); j++)
                                {
                                    Console.WriteLine("\tStatement type: " + ((TCustomSqlStatement)ifstmt._else_stmt_list[j]).SqlStatementType);
                                    Console.WriteLine("\tStatement text: " + ((TCustomSqlStatement)ifstmt._else_stmt_list[j]).AsText);
                                }
                            }

                            break;
                        default:
                            break;
                    }
                }
            }
        }
    }
}
